package ld26;

import com.ixee.GraphIX.*;
import java.awt.Color;
import com.ixee.datastruct.*;
import java.util.*;

public class HomeNode extends Node implements Home{
	Color homeCol;

	HashMap<Color, LinkedList<Color>> resources;

	int heldResources;
	long totTime = 0;

	long cycleTimeNs;
	long timeToNextResource;

	public HomeNode(Coordinate c){
		//c should always be (0, 0), yes?
		loc = c;

		heldResources = 0;

		cycleTimeNs = 2000*1000000L;
		timeToNextResource = cycleTimeNs;

		resources = new HashMap<Color, LinkedList<Color>>();

		product = new RBTree<Integer, LinkedList<Resource>>(false);
		radius = 50;
		connections = new HashSet<Node>();

		homeCol = new Color(120, 120, 150);
	}
	public void link(Node n){
		if(!this.linked(n)){
			connections.add(n);
		}
	}
	public void draw(GIXEngine e){
		if(active){
			e.graphics().drawCircle(loc, (float)radius*15, 0, 10, new Color(homeCol.getRed()/255f, homeCol.getGreen()/255f, homeCol.getBlue()/255f, 100/255f));
		}
		//System.out.println("Home depth: "+e.graphics().getDepth());
		e.graphics().drawCircle(loc, (float)radius, 0, 10, homeCol);

		e.graphics().drawString(Long.toString(totTime/1000000000L), loc.getX()+20, loc.getY(), new Color(190, 100, 120), e.graphics().JUSTIFICATION_LEFT);

		List<List<Color>> resources = this.resources();

		for(int i=0;i<resources.size();i++){
			if(resources.get(i).size() > 0){
				e.graphics().drawString(Integer.toString(resources.get(i).size()), loc.getX(), loc.getY()+12*i, resources.get(i).get(0), e.graphics().JUSTIFICATION_RIGHT);
			}
		}

		//drawLines(e);
	}
	public double getRadius(){
		return radius;
	}
	public double getActualRadius(){
		return radius;
	}
	private void produceResource(){
		Integer priority = (int)(cycleTimeNs/1000000L);
		Color[] colors = new Color[]{
					new Color(180, 60, 30),
					new Color(120, 150, 20),
					new Color(80, 50, 120),
					new Color(40, 180, 120),
		};
		//System.out.println(nodeColor+", "+priority);
		if(product.get(priority) == null){
			product.insert(priority, new LinkedList<Resource>());
		}
		for(int i=0;i<colors.length;i++){
			product.get(priority).add(new Resource(colors[i], priority));
		}
		timeToNextResource += cycleTimeNs;
	}
	public int msToResource(){
		return (int)(timeToNextResource/1000000L);
	}
	public void update(long ns){
		//set this up too.
		totTime += ns;

		timeToNextResource -= ns;

		while(timeToNextResource < 0){
			produceResource();
		}

		if(totTime/1000000 > 1500){
			//System.out.println("Doing pull..");
			main.playSound("pulse");
			pull();

			totTime -= 1500*1000000L;
		}
		while(product.size() > 0){
			LinkedList<Resource> curr = product.getMin();
			while(curr.size() > 0){
				store(curr.poll().getColor());
			}
			product.removeMin();
		}
	}
	public List<Color> resourceTypes(){
		Set<Map.Entry<Color, LinkedList<Color>>> entries = resources.entrySet();
		ArrayList<Color> ret = new ArrayList<Color>();

		for(Map.Entry<Color, LinkedList<Color>> entr : entries){
			ret.add(entr.getKey());
		}

		return ret;
	}
	public boolean store(Color c){
		if(!resources.containsKey(c)){
			resources.put(c, new LinkedList<Color>());
		}
		resources.get(c).add(c);
		heldResources++;
		return true;
	}
	public boolean remove(int n, Color c){
		if(!resources.containsKey(c)){
			return false;
		}else{
			LinkedList<Color> list = resources.get(c);
			if(list.size() < n){
				return false;
			}else{
				for(int i=0;i<n;i++){
					list.poll();
				}
				heldResources -= n;
				return true;
			}
		}
	}
	public boolean has(int n, Color c){
		if(!resources.containsKey(c)){
			return false;
		}else{
			return resources.get(c).size() >= n;
		}
	}
	public List<List<Color>> resources(){
		ArrayList<List<Color>> returnable = new ArrayList<List<Color>>();

		Set<Map.Entry<Color, LinkedList<Color>>> entries = resources.entrySet();

		for(Map.Entry<Color, LinkedList<Color>> entr : entries){
			returnable.add(entr.getValue());
		}

		return returnable;
	}
	public Color getColor(){
		return homeCol;
	}
}