package ld26;

import com.ixee.GraphIX.*;
import java.awt.Color;
import com.ixee.datastruct.*;
import java.util.*;

/*
		Perhaps minimalism is something interesting in and of itself.

		How little do you need to get something done?

		Humans are optimizing machines. We like finding optimal solutions.
		The least bad. The most good. The least slow. The most fast.

		So here we are optimizing away at game development.

		Can something you don't understand be optimized?

	What do we understand?


Let's put the fact that I do not understand game development aside.

		Let's assume I understand game development.

		Let's assume I am optimizing away at developing my game.
			This game.

		Does the video game reside entirely in this source code?


There is nothing to this game other than the source code?
		So there is something in the nature of this text that is entertaining?
		The meaning of this text, its organization.
		A meaning such that another piece of what c
		ould be described as text can take this tex
		t and translate it to another form. That fo
		rm is what is taken by more once-text organ
		izations and puts images and sounds on your
		display and in your ears.

		That doesn't sound very entertaining.

		I tend to find entertainment at Home.
*/

public class ResourceNode extends Node{
	Color nodeColor;

	long cycleTimeNs;

	long timeToNextResource;

	public ResourceNode(Color col, Coordinate c, int prodTimeMs){
		loc = c;
		cycleTimeNs = prodTimeMs*1000000L;
		timeToNextResource = cycleTimeNs;
		radius = Math.max((120000-prodTimeMs)/3000, 10);

		product = new RBTree<Integer, LinkedList<Resource>>(false);
		connections = new HashSet<Node>();

		nodeColor = col;
	}
	private void produceResource(){
		Integer priority = (int)(cycleTimeNs/1000000L);
		//System.out.println(nodeColor+", "+priority);
		if(product.get(priority) == null){
			product.insert(priority, new LinkedList<Resource>());
		}
		product.get(priority).add(new Resource(nodeColor, priority));
		timeToNextResource += cycleTimeNs;
	}
	public void link(Node n){
		if(!n.linked(this)){
			connections.add(n);
		}
	}
	public Color getColor(){
		return nodeColor;
	}
	public double getRadius(){
		glEEngine g = main.graphics();

		Coordinate c = g.getCam().getLocation();
		return (float)(Math.max(radius/c.getZ(), 15)*c.getZ());
	}
	public double getActualRadius(){
		return radius;
	}
	public void draw(GIXEngine e){
		glEEngine g = e.graphics();

		Coordinate c = g.getCam().getLocation();
		//System.out.println(radius);
		g.pushProjection();

//		g.translate(-loc.getX(), -loc.getY());
		float temprad = (float)(Math.max(radius/c.getZ(), 15)*c.getZ());
		int degPerSeg = Math.min(20, Math.max(10, (int)Math.round(c.getZ()*4)));
		g.drawCircle(loc, temprad, Math.min(temprad*0.95f, temprad-4), degPerSeg, nodeColor);

		temprad = Math.min(temprad*0.95f, temprad-4);
		g.drawCircle(loc, temprad, temprad-2*(float)c.getZ(), degPerSeg, Color.GRAY);

		if(active){
			g.drawCircle(loc, (float)radius*15, 0, 10, new Color(nodeColor.getRed()/255f, nodeColor.getGreen()/255f, nodeColor.getBlue()/255f, 100/255f));
		}
		//double[] d = new double[2];
		//d[0] = loc.getX();
		//d[1] = loc.getY();
		g.getCam().identity();
		g.translate(-c.getX(), -c.getY());
		g.translate(-loc.getX()/c.getZ(), -loc.getY()/c.getZ());
		//System.out.println(c.getZ());
		//g.getCam().project(d);
		//g.getCam().project(d);
		//g.getCam().invert();
		//g.getCam().scale(1/c.getZ(), 1/c.getZ());
//		g.getCam().identity();
//		g.getCam().translate(-c.getX(), -c.getY());
//		g.setCamLoc(new Coordinate(c.getX(), c.getY(), 1));

//		g.drawString(Long.toString((long)Math.ceil(timeToNextResource/1000000000.0)), loc.getX(), loc.getY(), nodeColor);
		g.drawString(Long.toString((long)Math.ceil(timeToNextResource/1000000000.0)), 0, 0, nodeColor);

		if(c.getZ() < 2){
			double r = Math.max(15, radius/c.getZ())+2;
			double angle = Math.toRadians(60);

			g.drawString("<"+Integer.toString(product.size())+">", r*Math.cos(angle), r*Math.sin(angle), nodeColor, glEEngine.JUSTIFICATION_LEFT);

			RBKeyIterator<Integer, Object> iter = (RBKeyIterator<Integer, Object>)product.keyIterator();

			int i = 0;

			while(iter.hasNext()){
				i++;
				angle -= Math.toRadians(30/r*20);
				Integer key = iter.next();
				g.drawString(": "+Integer.toString(product.get(key).size()), r*Math.cos(angle), r*Math.sin(angle), product.get(key).get(0).getColor(), glEEngine.JUSTIFICATION_LEFT);
			}

		}

		g.popProjection();
		//drawLines(e);

	}
	public int msToResource(){
		return (int)(timeToNextResource/1000000L);
	}
	public void update(long ns){
		timeToNextResource -= ns;

		while(timeToNextResource < 0){
			produceResource();
		}

		//do something about polling other nodes for resources, too. hm.
	}
}