/*
	This class defines an archive of BufferedImage, intended to be used to
	be loaded into the GraphIX engine as a glEAnim resource or be saved as a
	.gixanim file to be read by this class and later loaded as a glEAnim resource.
*/

package com.ixee.GraphIX;

import com.ixee.dataops.BitOperations;
import com.ixee.io.MarkedInputStream;
import com.ixee.io.MarkedOutputStream;
import com.ixee.io.FileLoader;
import com.ixee.io.stdio;
//import com.ixee.GraphIX.glEImage;
import com.ixee.GraphIX.glEAnim;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;

public class glEAnimArchive{
	BufferedImage[] images;
	byte version;
	int imageSizes[];
	int msPerFrame;
	int defaultLoops;
	String archiveName;
	String archiveLoc;
	public glEAnimArchive(int numImages){
		version = 02;
		images = new BufferedImage[numImages];
		msPerFrame = (int)(1000.0/numImages);
		defaultLoops = 0;
		archiveName = "";
	}
	public glEAnimArchive(String archiveLocation){
		if(!FileLoader.exists(archiveLocation)){
			stdio.debugAndLog(this, "File requested was not found! "+archiveLocation);
		}else{
			archiveLoc = archiveLocation;

			//stdio.log(this, "Loading and processing glEAnimArchive "+archiveLocation);
			//images = new BufferedImage[numImages];
			//for(int i=0;i<images.length;i++){
			//	ByteArrayInputStream bais = new ByteArrayInputStream(inputBytes.get(imageSizes[i]));
			//	try{
			//		images[i] = ImageIO.read(bais);
			//		stdio.debug(this, "Read an image, dimensions: "+images[i].getWidth(null)+", "+images[i].getHeight(null));
			//	}catch(IOException ioe){
			//		ioe.printStackTrace();
			//	}
			//}
		}
	}
	public glEAnim load(glEEngine e){
		//glEImage[] images = new glEImage[images.length];
		//for(int i=0;i<images.length;i++){
		//	images[i] = e.load(
		return e.load(this);
	}
	public void loadArchiveHeader(){
		stdio.debugAndLog("Loading header!");
		MarkedInputStream inputBytes = new MarkedInputStream(FileLoader.loadFile(archiveLoc));
		version = inputBytes.get();
		stdio.debugAndLog("Version: "+version);
		byte numImages = inputBytes.get();
		stdio.debugAndLog("Frame count: "+numImages);
		defaultLoops = inputBytes.get();
		stdio.debugAndLog("Num loops: "+defaultLoops);
		byte numNameChars = inputBytes.get();
		archiveName = new String(inputBytes.get(numNameChars));
		System.out.println("Name: "+archiveName);
		if(version == 01){
			msPerFrame = BitOperations.toInt(BitOperations.swapEndianness(inputBytes.get(4)));
			imageSizes = new int[numImages];
			for(int i=0;i<numImages;i++){
				imageSizes[i] = BitOperations.toInt(BitOperations.swapEndianness(inputBytes.get(4)));
			}
			images = new BufferedImage[numImages];
		}else{
			msPerFrame = BitOperations.toInt(inputBytes.get(4));
			imageSizes = new int[numImages];
			for(int i=0;i<numImages;i++){
				imageSizes[i] = BitOperations.toInt(inputBytes.get(4));
			}
			images = new BufferedImage[numImages];
		}
	}
	public static glEAnim loadArchive(String s, glEEngine e){
		return e.load(new glEAnimArchive(s));
	}
	public byte[] asBytes(){
		MarkedOutputStream mos = new MarkedOutputStream();
		mos.write(version);
		mos.write((byte)images.length);
		mos.write((byte)defaultLoops);
		byte[] bytes = archiveName.getBytes();
		mos.write((byte)bytes.length);
		mos.write(bytes);
		mos.write(BitOperations.toByteArray(msPerFrame));
		byte[][] imageBytes = new byte[images.length][];
		for(int i=0;i<images.length;i++){
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			try{
				ImageIO.write(images[i], "PNG", baos);
			}catch(IOException ioe){
				ioe.printStackTrace();
			}
			imageBytes[i] = baos.toByteArray();
		}
		for(int i=0;i<images.length;i++){
			mos.write(BitOperations.toByteArray(imageBytes[i].length));
		}
		for(int i=0;i<images.length;i++){
			mos.write(imageBytes[i]);
		}
		return mos.getBuffer();
	}
	public void setImage(int i, BufferedImage img){
		images[i] = img;
	}
	public BufferedImage getImage(int index){
		if(imageSizes == null){
			loadArchiveHeader();
		}
		BufferedImage returnable = null;
		int bytesToSkip = 0; 	//this will get set to the header plus the sum of sizes of images less than i
		bytesToSkip = 8 + archiveName.length();
		bytesToSkip += 4*imageSizes.length;
		for(int i=0;i<index;i++){
			bytesToSkip += imageSizes[i];
		}
		MarkedInputStream inputBytes = new MarkedInputStream(FileLoader.loadFile(archiveLoc));
		inputBytes.skip(bytesToSkip);
		ByteArrayInputStream bais = new ByteArrayInputStream(inputBytes.get(imageSizes[index]));
		try{
			returnable = ImageIO.read(bais);
		//	stdio.debug(this, "Read an image, dimensions: "+images[i].getWidth(null)+", "+images[i].getHeight(null));
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
		return returnable;
	}
	public BufferedImage[] getImages(){
		return images;
	}
	public int numFrames(){
		if(images == null){
			loadArchiveHeader();
		}
		return images.length;
	}
	public int getMsPerFrame(){
		return msPerFrame;
	}
	public int getDefaultLoops(){
		return defaultLoops;
	}
	public String name(){
		return archiveName;
	}
	public void setMsPerFrame(int ms){
		msPerFrame = ms;
	}
	public void setDefaultLoops(int i){
		defaultLoops = i;
	}
	public void setName(String s){
		archiveName = s;
	}
}