package com.ixee.GraphIX;

import com.ixee.datastruct.Coordinate;
import java.util.*;

public class glEMatrix{
	static double[] batchProjectBuffer = new double[4];
	double[] matrix;
	double[] intermediate;
	LinkedList<double[]> matrixStack;

	public static void main(String[] args){
		glEMatrix translate = new glEMatrix();
		translate.scale(0.5f, 0.5f);
		translate.translate(300, 300);
		translate.print();

		print(translate.project(0, 0));

		translate.rotate((float)Math.toRadians(45));
		translate.print();

		print(translate.project(32, 32));
	}
	public glEMatrix(){
		matrixStack = new LinkedList<double[]>();
		matrix = identity();
		intermediate = new double[9];
	}
	public static void print(double[] data){
		for(int i=0;i<data.length;i++){
			System.out.print(data[i]+", ");
		}
		System.out.println("");
	}
	public void print(){
		System.out.println(this);
		System.out.println("Current matrix: ");
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				System.out.print(matrix[i*3+j]+", ");
			}
			System.out.println("");
		}
		System.out.println("");
	}
	public void push(){
		double[] newMatrix = Arrays.copyOf(matrix, matrix.length);
		matrixStack.push(matrix);
		matrix = newMatrix;
	}
	public void pop(){
		matrix = matrixStack.pop();
	}
	public static double[] identity(){
		return new double[]{1, 0, 0, 0, 1, 0, 0, 0, 1};
	}
	public double[] getMatrix(){
		return matrix;
	}
	public void mult(glEMatrix t){
		mult(t.getMatrix());
	}
	public glEMatrix rotate(double d){
		mult(new double[]{Math.cos(d), -Math.sin(d), 0, Math.sin(d), Math.cos(d), 0, 0, 0, 1});
		return this;
	}
	public glEMatrix translate(Coordinate c){
		translate(c.getX(), c.getY());
		return this;
	}
	public glEMatrix translate(double x, double y){
		mult(new double[]{1, 0, 0, 0, 1, 0, -x, -y, 1});
		return this;
	}
	public glEMatrix scale(double x, double y){
		mult(new double[]{x, 0,0, 0, y, 0, 0, 0, 1});
		return this;
	}
	private void mult(double[] transformMatrix){
		if(transformMatrix.length == 9){
			for(int i=0;i<3;i++){
				for(int j=0;j<3;j++){
					float accum = 0;
					for(int k=0;k<3;k++){
						accum += transformMatrix[k*3+j]*matrix[i*3+k];
					}
					intermediate[i*3+j] = accum;
				}
			}
			double[] temp = matrix;
			matrix = intermediate;
			intermediate = temp;
		}
	}

	public glEMatrix rotate_p(double d){
		mult_p(new double[]{Math.cos(d), -Math.sin(d), 0, Math.sin(d), Math.cos(d), 0, 0, 0, 1});
		return this;
	}
	public glEMatrix translate_p(double x, double y){
		mult_p(new double[]{1, 0, 0, 0, 1, 0, -x, -y, 1});
		return this;
	}
	public glEMatrix scale_p(double x, double y){
		mult_p(new double[]{x, 0,0, 0, y, 0, 0, 0, 1});
		return this;
	}
	private void mult_p(double[] transformMatrix){
		if(transformMatrix.length == 9){
			for(int i=0;i<3;i++){
				for(int j=0;j<3;j++){
					float accum = 0;
					for(int k=0;k<3;k++){
						accum += transformMatrix[i*3+k]*matrix[k*3+j];
					}
					intermediate[i*3+j] = accum;
				}
			}
			double[] temp = matrix;
			matrix = intermediate;
			intermediate = temp;
		}
	}

	public double[] project(double x, double y){
		double[] result = new double[2];

		x = x*matrix[0] + y*matrix[3] + matrix[6];
		y = x*matrix[1] + y*matrix[4] + matrix[7];

		result[0] = x;
		result[1] = y;

		return result;
	}
	public double[] project(double[] result){
		double x, y;

		x = result[0]*matrix[0] + result[1]*matrix[3] + matrix[6];
		y = result[0]*matrix[1] + result[1]*matrix[4] + matrix[7];

		result[0] = x;
		result[1] = y;

		return result;
	}
	public double[] batchProject(double[] result, int stride){
		if(result.length%stride != 0){
			return result;
		}else{
			for(int i=0;i<result.length;i+=stride){
				batchProjectBuffer[0] = result[i]*matrix[0] + result[i+1]*matrix[3] + matrix[6];
				batchProjectBuffer[1] = result[i]*matrix[1] + result[i+1]*matrix[4] + matrix[7];

				result[i] = batchProjectBuffer[0];
				result[i+1] = batchProjectBuffer[1];
			}
		}
		return result;
	}
	public static glEMatrix composite(glEMatrix A, glEMatrix B){
		glEMatrix returnable = new glEMatrix();
		returnable.mult(A);
		returnable.mult(B);
		return returnable;
	}
	public void load(double[] newData){
		if(newData.length == 9){
			System.arraycopy(newData, 0, matrix, 0, matrix.length);
		}
	}
}