/*
	2013-04-07
		Altered glERenderVertex to include information on what texture
		is associated with this glERenderVertex.

		This is purely to support MODE_DRAW_BY_REQUEST in glEEngine, to
		allow unified use of glERenderVertex in listing all requests
		by order each request was placed.


		Also added information tracking how many vertices are in this request.
		Defaults to 4 as the use of this by and large is to draw quads.

		When I get around to properly reimplementing fillPoly/drawPoly
		to use the deferred rendering system they will need that ability though.
*/

package com.ixee.GraphIX;

public class glERenderVertex{
	float[] vertex;
	float[] color;
	float[] texture;

	int texID;

	int vertexCount;

	glERenderVertex link;

	public glERenderVertex(){
		vertex = new float[3];
		color = new float[4];
		texture = new float[2];

		texID = -1;
		vertexCount = 4;
	}
	public void load(double[] newFloat, float[] newColor, double depth){
		vertex[0] = (float)newFloat[0];
		vertex[1] = (float)newFloat[1];
		vertex[2] = (float)depth;

		color = newColor;
		texID = -1;
	}
	public void load(double x, double y, float[] newColor, double depth){
		vertex[0] = (float)x;
		vertex[1] = (float)y;
		vertex[2] = (float)depth;

		color = newColor;
		texID = -1;
	}
	public void load(double x, double y, float[] newColor, double depth, float u, float v){
		vertex[0] = (float)x;
		vertex[1] = (float)y;
		vertex[2] = (float)depth;

		color = newColor;

		texture[0] = u;
		texture[1] = v;
		texID = -1;
	}
	public void load(double x, double y, float r, float g, float b, float a, double depth, float u, float v){
		vertex[0] = (float)x;
		vertex[1] = (float)y;
		vertex[2] = (float)depth;

		color[0] = r;
		color[1] = g;
		color[2] = b;
		color[2] = a;

		texture[0] = u;
		texture[1] = v;
		texID = -1;
	}
	public void setTex(int tex){
		texID = tex;
	}
	public int getTex(){
		return texID;
	}

	public float[] getVertexCoords(){
		return vertex;
	}
	public float[] getColorCoords(){
		return color;
	}
	public float[] getTextureCoords(){
		return texture;
	}
	public void link(glERenderVertex next){
		link = next;
	}
	public glERenderVertex getLink(){
		return link;
	}
	public void unlink(){
		link = null;
	}
}