/*
	If any of these methods are provided an output array of size smaller than expected,
	the low-value bytes are discarded. If an array of size larger than needed, it is
	filled from index 0.
*/

package com.ixee.dataops;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Random;
import java.util.ArrayList;

public class BitOperations{
	public static char[] hexCharacters = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
	public static ArrayList<Character> hexChars = new ArrayList<Character>(Arrays.asList('0','1','2','3','4','5','6','7','8', '9', 'a', 'b', 'c', 'd', 'e', 'f'));

	public static void main(String[] args){
		int numRounds = 1000000;
		int numTests = 10;

		long testLong = 123456787654321L;
		int testInt = 43211234;
		short testShort = 129;
		byte testByte = 115;

		long longTest = 0;
		int intTest = 0;
		short shortTest = 0;
		byte byteTest = 0;

		System.out.println("Long-related tests: ");

		System.out.println("Test not written yet. Please do so when you get around to it");

		System.out.println("");
		System.out.println("Int-related tests: ");
		System.out.println(" Hardcoded int value: "+testInt);
		byte[] intbytes = toByteArray(testInt);
		System.out.print("   As bytes: ");
		for(int i=0;i<intbytes.length;i++){
			System.out.print(intbytes[i]+" " );
		}
		System.out.println("");
		intTest = BitOperations.toInt(intbytes);
		System.out.println("   And as an integer again: "+intTest);

		System.out.println("\n   Pass the test? "+(intTest == testInt));

		System.out.println("");
		System.out.println("Short-related tests: ");
		System.out.println(" Hardcoded short value: "+testShort);
		byte[] shortbytes = toByteArray(testShort);
		System.out.print("   As bytes: ");
		for(int i=0;i<shortbytes.length;i++){
			System.out.print(shortbytes[i]+" " );
		}
		System.out.println("");
		shortTest = BitOperations.toShort(shortbytes);
		System.out.println("   And as a short again: "+shortTest);

		System.out.println("\n   Pass the test? "+(shortTest == testShort));

		System.out.println("");
		System.out.println("Byte-related tests: ");
		System.out.println(" Hardcoded byte value: "+testByte);
		System.out.println("");


		Random numberGen = new Random(0);
	}
		//simply converts the input to an array of bytes and
		//calls swapEndianness() on it
	public static byte[] swapEndianness(int setOfBytes){
		return swapEndianness(toByteArray(setOfBytes));
	}


		//Swaps the endianness of the input array of bytes
		//takes byte[i], copies it to an internal byte,
		//moves byte[byte.length-i] to byte[i]
		//then copies the internal byte to byte[byte.length-1]

		//proceeds from i=0 to i=byte.size/2, then returns the resultant array.
	public static byte[] swapEndianness(byte[] toSwap){
		byte[] internalBuffer = toSwap;
		int size = toSwap.length;
		byte buffer;
		for(int i=0;i<size/2;i++){
			buffer = internalBuffer[i];
			internalBuffer[i] = internalBuffer[size-(i+1)];
			internalBuffer[size-(i+1)] = buffer;
		}
		return internalBuffer;
	}

		//Converts the eight bytes in a double to a byte array
	public static byte[] toByteArray(double d){
		byte[] returnable;

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bos);

		try{
			dos.writeDouble(d);
		}catch(IOException ioe){}

		returnable = bos.toByteArray();

		return returnable;
	}
		//Converts the four bytes in a float to a byte array
	public static byte[] toByteArray(float d){
		byte[] returnable;

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bos);

		try{
			dos.writeFloat(d);
		}catch(IOException ioe){}

		returnable = bos.toByteArray();

		return returnable;
	}
		//Converts the two bytes in a short to a byte array
	public static byte[] toByteArray(short s){
		byte[] returnable = new byte[2];
		toByteArray(s, returnable);
		return returnable;
	}

		//Converts the four bytes in an int to a byte array
	public static byte[] toByteArray(int i){
		byte[] returnable = new byte[4];
		toByteArray(i, returnable);
		return returnable;
	}

	public static byte[] toByteArray(long l){
		return toByteArray(l, new byte[8]);
	}
	public static byte[] toByteArray(long l, byte[] b){
		return toByteArray(l, b, 0);
	}
	public static byte[] toByteArray(long l, byte[] b, int offset){
		long mask = 0xFFL;
		int max = Math.min(b.length-offset, 8);
		for(int i=0;i<max;i++){
			b[i+offset] = (byte)((l & mask)>>(i*8));
			mask = mask << 8;
		}
		return b;
	}

	public static long toLong(byte[] b){
		long returnable = 0x0000000000000000L;

		returnable = ((long)(b[7]&0xff))<<56|((long)(b[6]&0xff))<<48|((long)(b[5]&0xff))<<40|((long)(b[4]&0xff))<<32|((long)(b[3]&0xff))<<24|((long)(b[2]&0xff))<<16|((long)(b[1]&0xff))<<8|((long)(b[0]&0xff));

		return returnable;
	}


		//Converts an array (hopefully, of four bytes) to an int
	public static int toInt(byte[] b){
		int returnable = 0x00000000;

		returnable = (b[3]&0xff)<<24|(b[2]&0xff)<<16|(b[1]&0xff)<<8|(b[0]&0xff);

		return returnable;
	}

		//Converts an array (hopefully, of two bytes) to a short
	public static short toShort(byte[] b){
		short returnable = 0x0000;

		returnable = (short)(((b[1]<<8)&0xFF00)|(b[0]&0xFF));

		return returnable;
	}
		//Converts an array (hopefully of eight bytes) to a double
	public static double toDouble(byte[] b){
		double returnable = Double.NaN;

		ByteArrayInputStream bis = new ByteArrayInputStream(b);
		DataInputStream dis = new DataInputStream(bis);

		try{
			returnable = dis.readDouble();
		}catch(IOException ioe){}

		return returnable;
	}
		//Converts an array (hopefully of eight bytes) to a double
	public static float toFloat(byte[] b){
		float returnable = Float.NaN;

		ByteArrayInputStream bis = new ByteArrayInputStream(b);
		DataInputStream dis = new DataInputStream(bis);

		try{
			returnable = dis.readFloat();
		}catch(IOException ioe){}

		return returnable;
	}
	public static boolean[] toBooleanBits(byte[] b){
		boolean[] returnable = new boolean[b.length*8];
		byte[] toWorkOn = b;
		for(int i=0;i<b.length;i++){
			for(int inc=0;inc<8;inc++){
				if((0x01 & toWorkOn[i]) == 1){
					returnable[inc+i*8] = true;
				}else{
					returnable[inc+i*8] = false;
				}
				toWorkOn[i] = (byte)(toWorkOn[i]>>>0x01);
			}
		}
		return returnable;
	}
	public static byte setBit(byte b, int index){
		byte returnable = b;
		byte mask=0x00;
		if(index<8 && index >= 0){
			mask = (byte)(0x01<<index);
		}
		returnable = (byte)(returnable|mask);
		return returnable;
	}
	public static boolean hasBit(byte b, int index){
		int operable = b;
		int mask = 0x00;
		if(index<8 && index>=0){
			mask = 0x01<<index;
			operable = operable&mask;
			operable = operable>>>index;
			boolean returnable = false;
			if(operable == 1){
				returnable = true;
			}else{
				returnable = false;
			}
			return returnable;
		}else{
			System.out.println("BitOperations.132]: Index out of bounds. Put an exception here.");
			return false;
		}
	}
	public static String bytesToString(byte[] input){
		StringBuffer returnable = new StringBuffer();
		for(int i=0;i<input.length;i++){
			returnable.append(hexCharacters[(input[i] >> 4) & 0x0F]);
			returnable.append(hexCharacters[input[i] & 0x0F]);
		}
		return returnable.toString();
	}
	public static byte[] readHexString(String s){
		byte[] returnable = new byte[(int)Math.ceil(s.length()/2.0)];
		for(int i=0;i<s.length()/2;i++){
			returnable[i] = (byte)(hexChars.indexOf(s.charAt(i*2)));
			if(i*2+1 < s.length()){
				returnable[i] = (byte)(returnable[i] << 4);
				returnable[i] += hexChars.indexOf(s.charAt(i*2+1));
			}
		}
		return returnable;
	}
}