/*
	Coordi3 provides a three dimensional coordinate object that is backed by integer values.

	Thus, it operates on fixed-point math. The precision of x/y/z is
	fixed at 36 bits.28 bits
*/

package com.ixee.datastruct;

public class Coordi3{
	long x, y, z;

	final static int DECIMAL_PREC = 30;		//in bits. Though really
										//precision is only guaranteed to
										//DECIMAL_PREC/2

	public static void main(String[] args){
		Coordi3 a = new Coordi3(1, 2, 3);
		System.out.println(a);
		a.add(new Coordi3(6, 7, 8).div(3).mul(2));
		System.out.println(a);
		a.mulu(3.5);
		System.out.println(a);
		System.out.println(a.mul(3));
	}

	public Coordi3(long x, long y){
		this.x = x<<DECIMAL_PREC;
		this.y = y<<DECIMAL_PREC;
	}
	public Coordi3(long x, long y, long z){
		this.x = x<<DECIMAL_PREC;
		this.y = y<<DECIMAL_PREC;
		this.z = z<<DECIMAL_PREC;
	}
	public Coordi3(double x, double y){
		this.x = (long)(x*(1<<DECIMAL_PREC));
		this.y = (long)(y*(1<<DECIMAL_PREC));
		this.z = 0;
	}
	public Coordi3(double x, double y, double z){
		this.x = (long)(x*(1<<DECIMAL_PREC));
		this.y = (long)(y*(1<<DECIMAL_PREC));
		this.z = (long)(z*(1<<DECIMAL_PREC));
	}
	public Coordi3(Coordi3 c){
		x = c.x_raw();
		y = c.y_raw();
		z = c.z_raw();
	}
	public double getX(){
		return ((double)x)/(1<<DECIMAL_PREC);
	}
	public double getY(){
		return ((double)y)/(1<<DECIMAL_PREC);
	}
	public double getZ(){
		return ((double)z)/(1<<DECIMAL_PREC);
	}
	public long intX(){
		return x>>DECIMAL_PREC;
	}
	public long intY(){
		return y>>DECIMAL_PREC;
	}
	public long intZ(){
		return z>>DECIMAL_PREC;
	}
	public long x_raw(){
		return x;
	}
	public long y_raw(){
		return y;
	}
	public long z_raw(){
		return z;
	}
	public Coordi3 add_n(long a, long b, long c){
		return new Coordi3(this).add(a, b, c);
	}
	public Coordi3 add(long a, long b, long c){
		x += a << DECIMAL_PREC;
		y += b << DECIMAL_PREC;
		z += c << DECIMAL_PREC;
		return this;
	}
	public Coordi3 add_n(Coordi3 c){
		return new Coordi3(this).add(c);
	}
	public Coordi3 add(Coordi3 c){
		this.x += c.x_raw();
		this.y += c.y_raw();
		this.z += c.z_raw();
		return this;
	}
	public Coordi3 sub_n(long a, long b, long c){
		return new Coordi3(this).add(a, b, c);
	}
	public Coordi3 sub(long a, long b, long c){
		x -= a << DECIMAL_PREC;
		y -= b << DECIMAL_PREC;
		z -= c << DECIMAL_PREC;
		return this;
	}
	public Coordi3 sub_n(Coordi3 c){
		return new Coordi3(this).sub(c);
	}
	public Coordi3 sub(Coordi3 c){
		this.x -= c.x_raw();
		this.y -= c.y_raw();
		this.z -= c.z_raw();
		return this;
	}

	public Coordi3 mul_n(long i){
		return new Coordi3(this).mul(i);
	}
	public Coordi3 mul(long l){
		x = x*l;
		y = y*l;
		z = z*l;
		return this;
	}
	public Coordi3 mulu_n(double d){
		return new Coordi3(this).mulu(d);
	}
	public Coordi3 mulu(double d){
		x = ( x >> (DECIMAL_PREC/2))*(long)(d*(1<<(DECIMAL_PREC/2)));
		y = ( y >> (DECIMAL_PREC/2))*(long)(d*(1<<(DECIMAL_PREC/2)));
		z = ( z >> (DECIMAL_PREC/2))*(long)(d*(1<<(DECIMAL_PREC/2)));
		return this;
	}
	public Coordi3 mul_n(IDec d){
		return new Coordi3(this).mul(d);
	}
	public Coordi3 mul(IDec d){
		x = ( x >> (DECIMAL_PREC/2))*(d.val_raw()>>(DECIMAL_PREC/2));
		y = ( y >> (DECIMAL_PREC/2))*(d.val_raw()>>(DECIMAL_PREC/2));
		z = ( z >> (DECIMAL_PREC/2))*(d.val_raw()>>(DECIMAL_PREC/2));
		return this;
	}
	public Coordi3 divu_n(double d){
		return new Coordi3(this).mulu(d);
	}
	public Coordi3 divu(double d){
		x = ( x >> (DECIMAL_PREC/2))*(long)((1/d)*(1<<(DECIMAL_PREC/2)));
		y = ( y >> (DECIMAL_PREC/2))*(long)((1/d)*(1<<(DECIMAL_PREC/2)));
		z = ( z >> (DECIMAL_PREC/2))*(long)((1/d)*(1<<(DECIMAL_PREC/2)));
		return this;
	}
	public Coordi3 div_n(long l){
		return new Coordi3(this).div(l);
	}
	public Coordi3 div(long l){
		x = x/l;
		y = y/l;
		z = z/l;
		return this;
	}
	public double magnitude(){
		long tot;
		tot  = ( x >> (DECIMAL_PREC/2))*( x >>(DECIMAL_PREC/2));
		tot += ( y >> (DECIMAL_PREC/2))*( y >>(DECIMAL_PREC/2));
		tot += ( z >> (DECIMAL_PREC/2))*( z >>(DECIMAL_PREC/2));

		return Math.sqrt(new Double(tot));
	}
	public String toString(){
		StringBuilder ret = new StringBuilder();
		ret.append( Double.toString( new Double(x) / (1<<DECIMAL_PREC) ) );
		ret.append(", ");
		ret.append( Double.toString( new Double(y) / (1<<DECIMAL_PREC) ) );
		ret.append(", ");
		ret.append( Double.toString( new Double(z) / (1<<DECIMAL_PREC) ) );
		return ret.toString();
	}
}