package com.ixee.font;

import java.awt.image.BufferedImage;

public class FontDataAccelerated implements IxFont{
	String[] charDescs = new String[128];
	int[] glyphLocations = new int[128];
	int imageWidth;
	int imageHeight;
	int fontSetAddress;
	byte charWidth;
	boolean monospace;
	BufferedImage glyphSet;

	public FontDataAccelerated(){
		monospace = true;
		charDescs[0] = "[NUL] null character";
		charDescs[1] = "[SOH] start of heading";
		charDescs[2] = "[STX] start of text";
		charDescs[3] = "[EXT]";
		charDescs[4] = "[EOT]";
		charDescs[5] = "[ENQ]";
		charDescs[6] = "[ACK]";
		charDescs[7] = "[BEL]";
		charDescs[8] = "[BS]";
		charDescs[9] = "[TAB]";
		charDescs[10] = "[LF]";
		charDescs[11] = "[VT]";
		charDescs[12] = "[FF]";
		charDescs[13] = "[CR]";
		charDescs[14] = "[SO]";
		charDescs[15] = "[SI]";
		charDescs[16] = "[DLE]";
		charDescs[17] = "[DC1]";
		charDescs[18] = "[DC2]";
		charDescs[19] = "[DC3]";
		charDescs[20] = "[DC4]";
		charDescs[21] = "[NAK]";
		charDescs[22] = "[SYN]";
		charDescs[23] = "[ETB]";
		charDescs[24] = "[CAN]";
		charDescs[25] = "[EM]";
		charDescs[26] = "[SUB]";
		charDescs[27] = "[ESC]";
		charDescs[28] = "[FS]";
		charDescs[29] = "[GS]";
		charDescs[30] = "[RS]";
		charDescs[31] = "[US]";
		charDescs[32] = "Space";
		charDescs[33] = "!";
		charDescs[34] = "quotation mark";
		charDescs[35] = "#";
		charDescs[36] = "$";
		charDescs[37] = "%";
		charDescs[38] = "&";
		charDescs[39] = "'";
		charDescs[40] = "(";
		charDescs[41] = ")";
		charDescs[42] = "*";
		charDescs[43] = "+";
		charDescs[44] = ",";
		charDescs[45] = "-";
		charDescs[46] = ".";
		charDescs[47] = "/";
		charDescs[48] = "0";
		charDescs[49] = "1";
		charDescs[50] = "2";
		charDescs[51] = "3";
		charDescs[52] = "4";
		charDescs[53] = "5";
		charDescs[54] = "6";
		charDescs[55] = "7";
		charDescs[56] = "8";
		charDescs[57] = "9";
		charDescs[58] = ":";
		charDescs[59] = ";";
		charDescs[60] = "<";
		charDescs[61] = "=";
		charDescs[62] = ">";
		charDescs[63] = "?";
		charDescs[64] = "@";
		charDescs[65] = "A";
		charDescs[66] = "B";
		charDescs[67] = "C";
		charDescs[68] = "D";
		charDescs[69] = "E";
		charDescs[70] = "F";
		charDescs[71] = "G";
		charDescs[72] = "H";
		charDescs[73] = "I";
		charDescs[74] = "J";
		charDescs[75] = "K";
		charDescs[76] = "L";
		charDescs[77] = "M";
		charDescs[78] = "N";
		charDescs[79] = "O";
		charDescs[80] = "P";
		charDescs[81] = "Q";
		charDescs[82] = "R";
		charDescs[83] = "S";
		charDescs[84] = "T";
		charDescs[85] = "U";
		charDescs[86] = "V";
		charDescs[87] = "W";
		charDescs[88] = "X";
		charDescs[89] = "Y";
		charDescs[90] = "Z";
		charDescs[91] = "[";
		charDescs[92] = "backslash";
		charDescs[93] = "]";
		charDescs[94] = "^";
		charDescs[95] = "_";
		charDescs[96] = "`";
		charDescs[97] = "a";
		charDescs[98] = "b";
		charDescs[99] = "c";
		charDescs[100] = "d";
		charDescs[101] = "e";
		charDescs[102] = "f";
		charDescs[103] = "g";
		charDescs[104] = "h";
		charDescs[105] = "i";
		charDescs[106] = "j";
		charDescs[107] = "k";
		charDescs[108] = "l";
		charDescs[109] = "m";
		charDescs[110] = "n";
		charDescs[111] = "o";
		charDescs[112] = "p";
		charDescs[113] = "q";
		charDescs[114] = "r";
		charDescs[115] = "s";
		charDescs[116] = "t";
		charDescs[117] = "u";
		charDescs[118] = "v";
		charDescs[119] = "w";
		charDescs[120] = "x";
		charDescs[121] = "y";
		charDescs[122] = "z";
		charDescs[123] = "{";
		charDescs[124] = "|";
		charDescs[125] = "}";
		charDescs[126] = "~";
		charDescs[127] = "[DEL]";
		for(int i=0;i<glyphLocations.length;i++){
			glyphLocations[i] = -1;
		}
	}
	public void setCharPosition(int loc, char c){
		glyphLocations[c] = loc;
	}
	public int charToLoc(char c){
		return glyphLocations[c];
	}
	public int getWidth(String s){
		return getWidth(s, 0, s.length());
	}
	public int getWidth(String s, int start, int end){
		if(monospace){
			return (end-start)*charWidth;
		}else{
			if(start < 0){
				throw new ArrayIndexOutOfBoundsException();
			}
			if(end > s.length()){
				throw new ArrayIndexOutOfBoundsException();
			}
			int returnable = 0;
			for(int i=start;i<end;i++){
				returnable += getWidth(s.charAt(i));
			}
			return returnable;	//DOES NOT WORK.
		}
	}
	public int getWidth(char c){
		int returnable = 0;
		int start = glyphLocations[c];
		if(start == -1){
			returnable = -1;
		}else{
			char nextGlyph = (char)(c+1);
			int nextGlyphLocation = glyphLocations[nextGlyph];
			while(nextGlyph != c && nextGlyphLocation == -1){
				nextGlyph = nextGlyph++;
				if(nextGlyph == 128){
					nextGlyphLocation = imageWidth;
				}
				nextGlyphLocation = glyphLocations[nextGlyph];
			}
			returnable = nextGlyphLocation-start;
		}
		//But what about the character at the last position in the table? Dunno.
		return returnable;
	}
	public int getHeight(){
		return imageHeight;
	}
	public int getHeight(char c){
		return getHeight();
	}
	public int getImageAddress(){
		return fontSetAddress;
	}
	public void setCharWidth(byte i){
		charWidth = i;
	}
	public byte getCharWidth(){
		return charWidth;
	}
	public void setImageAddress(int address){
		fontSetAddress = address;
	}
	public void setGlyphs(BufferedImage b){
		glyphSet = b;
		imageWidth = b.getWidth(null);
		imageHeight = b.getHeight(null);
	}
	public BufferedImage getImage(){
		return glyphSet;
	}
	public float getRatio(){
		return 1.0f/imageWidth;
	}
	//public int getPreciseHeight(char c){
		//haven't gotten around to figuring this one out quite yet. Such is life.
	//}
}