package com.ixee.font;

import com.ixee.dataops.BitOperations;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.ByteArrayInputStream;
import java.io.BufferedReader;
import java.io.FileReader;
import java.awt.image.BufferedImage;
import java.awt.Image;
import com.ixee.io.stdio;

public class FontLoader{
	/*
	first 1+128*4=513 bytes are header data.
	byte 1 is what version of font file the file to load is.
	*** Note: this example uses offsets as if they were three bytes. I changed them to four. ***
	each triplet of bytes afterward is a pointer to what offset in the file the associated character's image is.
	for example:
				01 00 01 5D 00 02 A1 00 00 00 ..
	would be a font header marking font version 01 (this), with the first character being at
	offset (hex)00015D(dec)349; immediately after the header. This is followed by the second character
	at 0002A1(dec)673. the third character is at 000000 or first byte, which is the version byte. this
	means that the third character has no associated image in this font and is to be passed over.

	According to ASCII, space is (dec)32, so I doubt any font used here will have anything for the first
	32 symbols but who knows.

	Getting images from a font file is simply reading from the offset to the next real offset (or end of file).
	*/
	/*
		2011-10-04

		Ixi strikes again!

		All the texture binding and re-binding associated with rendering strings takes too much time.
		Going to alter things a little so we have fonts for opengl use.
		These altered fonts will have one large image of all characters, and then offsets to where
		the characters themselves start.

		This means...
			Traditional Ixineering font files for general use have the ".ixf" extension
			Where as fonts for GraphIX use will have the ".gixf" extension.
			.gixf fonts will also have a 1 in the leftmost bit of the first byte ->
				all .gixf fonts will have a version of 128+x where x is the real verison number.

	*/
	public static String fontFilesPath = ".";
	static String defaultLoadable = "/defaultIn.ixf";
	static String defaultSaveable = "/defaultOut.ixf";
	static String defaultAcceleratedSaveable = "/defaultOut.gixf";
	static String defaultAcceleratedLoadable = "/defaultIn.gixf";

	static String defaultGlyphImageName = "/glyphs.png";

	public FontLoader(){
	}
	public void setPath(String newPath){
		fontFilesPath = newPath;
	}
	public static void writeFont(FontDataAccelerated accel){
		byte version = (byte)(0x80)+2;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try{
			ImageIO.write(accel.getImage(), "PNG", baos);
			byte[] imageBytes = baos.toByteArray();
			File output = new File("."+defaultAcceleratedSaveable);
			FileOutputStream fos = new FileOutputStream(output);
			fos.write(version);
			fos.write(accel.getCharWidth());
			for(char i=0;i<128;i++){
				fos.write(BitOperations.toByteArray(accel.charToLoc(i)));
			}
			fos.write(baos.toByteArray());
			fos.flush();
			fos.close();
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
	}

	public static void writeFont(FontData output){
		byte[][] charGlyphBytes = new byte[128][];
	//	int fontFileSize = 513;
		for(char i=0;i<128;i++){
			if(output.charToImage(i)!=null){
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				try{
					ImageIO.write((BufferedImage)output.charToImage(i), "PNG", baos);
				}catch(IOException ioe){
				}
				charGlyphBytes[i] = baos.toByteArray();
	//			fontFileSize+= charGlyphBytes[i].length;
			}else{
				charGlyphBytes[i] = new byte[0];
			}
		}
		try{
			File fontOut = new File(defaultSaveable);
			FileOutputStream fos = new FileOutputStream(fontOut);
			fos.write((byte)1);
			int offset = 513;
			for(char i=0;i<128;i++){
				fos.write(BitOperations.toByteArray(offset));
				offset += charGlyphBytes[i].length;
			}
			for(char i=0;i<128;i++){
				fos.write(charGlyphBytes[i]);
			}
			fos.flush();
			fos.close();
		}catch(IOException ioe){
		}
	}
	public static FontData makeFont(){
		FontData returnable = new FontData();

		for(int i=0;i<128;i++){
			File loadable = new File(fontFilesPath+"/"+i+".png");
			if(loadable.exists()){
				try{
					returnable.setChar(i, ImageIO.read(loadable));
				}catch(IOException ioe){
				}
			}
		}
		return returnable;
	}
	public static FontDataAccelerated makeAcceleratedFont(){
		return makeAcceleratedFont(fontFilesPath);
	}
	public static FontDataAccelerated makeAcceleratedFont(String path){
		return makeAcceleratedFont(path, (byte)8);
	}
	public static FontDataAccelerated makeAcceleratedFont(String path, byte charWidth){
		FontDataAccelerated returnable = new FontDataAccelerated();
		returnable.setCharWidth(charWidth);
		File glyphs = new File(path+"glyph.bmp");
		try{
			try{
				System.out.println("Loading image: "+glyphs.getCanonicalPath());
			}catch(IOException ioe){
				System.out.println("Failed to find glyph file at "+path+"glyph.bmp");
			}
			BufferedImage fixedTransparencies = ImageIO.read(glyphs);
			File glyphLocations = new File(path+"glyphLocations.txt");
			try{
				System.out.println(glyphLocations.getCanonicalPath());
			}catch(IOException ioe){
				ioe.printStackTrace();
				System.out.println("Couldn't find glyph location file!");
			}
			BufferedReader br = new BufferedReader(new FileReader(glyphLocations));
			String temp = br.readLine();
			int i=0;
			while(temp != null){
				System.out.println(temp);
				String[] parts = temp.split(" ");
				char index = (char)Integer.parseInt(parts[0]);
				int loc = Integer.parseInt(parts[1]);
				returnable.setCharPosition(loc, index);
				temp = br.readLine();
			}
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
		return returnable;
	}
	public static FontDataAccelerated loadAcceleratedFont(String path){
		//ISN'T READING IN THE IMAGE HURRR
		stdio.debugAndLog("Loading font: "+path);
		FontDataAccelerated returnable = new FontDataAccelerated();

		File fontFile = new File(path);
		byte[] fontBytes = new byte[(int)fontFile.length()];
		try{
			FileInputStream fis = new FileInputStream(fontFile);
			fis.read(fontBytes);
		}catch(IOException e){
			stdio.debugAndLog("Unable to locate font: " +path);
			e.printStackTrace();
		}
		int[] locations = new int[128];
		int currOffset = 0;
		currOffset += 1;
		if(fontBytes[0] < 0){
			if(fontBytes[0] == (byte)0x80+2){
				returnable.setCharWidth(fontBytes[currOffset]);
				currOffset++;
			}
			byte[] curr = new byte[4];
			for(char i=0;i<128;i++){
				System.arraycopy(fontBytes, currOffset, curr, 0, 4);
				returnable.setCharPosition(BitOperations.toInt(curr), i);
				currOffset += 4;
			}
			byte[] glyphImageBytes = new byte[fontBytes.length-currOffset];
			System.arraycopy(fontBytes, currOffset, glyphImageBytes, 0, glyphImageBytes.length);
			//stdio.debugAndLog("Copying "+glyphImageBytes.length);
			ByteArrayInputStream bais = new ByteArrayInputStream(glyphImageBytes);
			try{
				BufferedImage b = ImageIO.read(bais);
			//	stdio.debugAndLog("Setting font glyphs");
				returnable.setGlyphs(b);
			}catch(IOException ioe){
				ioe.printStackTrace();
			}
			return returnable;

		}else{
			stdio.debugAndLog("Invalid header. This file is marked as an .ixf, not .gixf");
			return null;
		}
	}
	public static FontData loadFont(String path){
		FontData returnable = new FontData();

		File fontFile = new File(path);
		byte[] fontBytes = new byte[(int)fontFile.length()];
		try{
			FileInputStream fontFIS = new FileInputStream(fontFile);
			fontFIS.read(fontBytes);
		}catch(IOException e){
			e.printStackTrace();
		}
		int[] offsets = new int[128];
		int currOffset = 0;
		byte[] offsetBytes = new byte[4];
		currOffset += 1; //moving past version byte.
		for(int i=0;i<128;i++){
			System.arraycopy(fontBytes, currOffset, offsetBytes, 0, 4);
			int offset = BitOperations.toInt(offsetBytes);
			offsets[i] = offset;
			currOffset+=4;
		}
		byte[] imageBytes;
		ByteArrayInputStream imageByte;
		for(int i=0;i<127;i++){
			if(offsets[i+1]-offsets[i] > 0){
				imageBytes = new byte[offsets[i+1]-offsets[i]];
				System.arraycopy(fontBytes, offsets[i], imageBytes, 0, imageBytes.length);
				imageByte = new ByteArrayInputStream(imageBytes);
				try{
					returnable.setChar(i, (Image)ImageIO.read(imageByte));
					imageByte.close();
				}catch(IOException ioe2){
					ioe2.printStackTrace();
				}
			}
		}
		if(fontBytes.length-offsets[127]>0){
			imageBytes= new byte[fontBytes.length-offsets[127]];
			System.arraycopy(fontBytes, offsets[127], imageBytes, 0, imageBytes.length);
			imageByte = new ByteArrayInputStream(imageBytes);
			try{
				returnable.setChar(127, (Image)ImageIO.read(imageByte));
				imageByte.close();
			}catch(IOException ioe1){
				ioe1.printStackTrace();
			}
		}
		return returnable;
	}
	public static FontData loadFromDefaultPath(){
		return loadFont(fontFilesPath+defaultLoadable);
	}
	public static FontDataAccelerated loadFromDefaultAcceleratedPath(){
		return loadAcceleratedFont(fontFilesPath+defaultAcceleratedLoadable);
	}

}