//stores information about the cursor. it's up to the application to
//update the information stored in a CursorInfo object.

//It assumes (0, 0) is the center of the screen.
package com.ixee.ui;

import com.ixee.datastruct.Coordinate;

public class CursorInfo{
	//
	public final static int LEFT = 0;
	public final static int RIGHT = 1;
	public final static int MIDDLE = 2;
	public final static Coordinate notOnScreen = new Coordinate(Double.NaN, Double.NaN, 0);
	protected Coordinate correctionFactor = new Coordinate(0,0,0);
	Coordinate pressLocs[];
	Coordinate releaseLocs[];
	boolean presses[];
	boolean clicked[];
	Coordinate location = new Coordinate(0, 0, 0);
	int mode = -1;
	boolean anyClick = false;

	public CursorInfo(Coordinate c, int numButtons){
		correctionFactor = c;
		pressLocs = new Coordinate[numButtons];
		releaseLocs = new Coordinate[numButtons];
		presses = new boolean[numButtons];
		clicked = new boolean[numButtons];

	}
	public void setPosition(Coordinate c){
		location = c;
	}
	public void setPress(Coordinate c, int button){
		presses[button] = true;
		pressLocs[button] = c;
	}
	public void setRelease(Coordinate c, int button){
		anyClick = true;
		presses[button] = false;
		clicked[button] = true;
		releaseLocs[button] = c;
	}
	public void clear(){
		anyClick = false;
		for(int i=0;i<presses.length;i++){
			presses[i] = false;
			clicked[i] = false;
			releaseLocs[i] = notOnScreen;
			pressLocs[i] = notOnScreen;
		}
	}
	public void clear(int i){
		presses[i] = false;
		releaseLocs[i] = notOnScreen;
		pressLocs[i] = notOnScreen;
	}
	public boolean pressed(int i){
		return presses[i];
	}
	public boolean clicked(){
		return anyClick;
	}
	public boolean clicked(int i){
		return clicked[i];
	}
	public boolean dragged(int i){
		boolean returnable = true;
		if(presses[i] || clicked[i]){
			if(clicked[i]){
				for(int j=0;j<2;j++){
					if(pressLocs[i].getInt(j) != releaseLocs[i].getInt(j)){
						returnable = false;
					}
				}
			}else{
				for(int j=0;j<2;j++){
					if(pressLocs[i].getInt(j) != location.getInt(j)){
						returnable = false;
					}
				}
			}
		}
		return !returnable;
	}
	public Coordinate pressLoc(int i){
		return pressLocs[i];
	}
	public Coordinate releaseLoc(int i){
		return releaseLocs[i];
	}
	public Coordinate position(){
		return location;
	}
	public void bindMode(int mod){
		mode = mod;
	}
	public int getMode(){
		return mode;
	}
	public void sync(Coordinate position, boolean[] p, boolean[] r){
		location = new Coordinate(position.getX(), -position.getY());
		for(int i=0;i<p.length;i++){
			if(presses[i] == false){
				pressLocs[i] = notOnScreen;
				if(p[i] == true){
					presses[i] = true;
					pressLocs[i] = location;
				}
			}
			presses[i] = p[i];
		}
		for(int i=0;i<r.length;i++){
			if(clicked[i] == false){
				releaseLocs[i] = notOnScreen;
				if(r[i] == true){
					clicked[i] = true;
					releaseLocs[i] = location;
				}
			}
			clicked[i] = r[i];
		}
		boolean clearAnyClicks = false;
		for(int i=0;i<r.length;i++){
			if(r[i]){
				clearAnyClicks = true;
			}
		}
		if(clearAnyClicks){
			anyClick = false;
		}
	}

}