/*
	Active elements are any UI elements that can react to a mouse or key press
	because of this, and the fact that we're trying to stay in form with the
	way windows behaves, we make every input-actable element have a "focus",
	and if a key is pressed, it grabs the focus of what elements would react
	to key input.
*/
package com.ixee.ui;

import com.ixee.datastruct.Coordinate;

public class UIActiveElement extends UIElement{
	protected int boundMode = -1;
	protected boolean toggleActiveMode;
	protected boolean active;

	public UIActiveElement(Coordinate position){
		super(position);
		toggleActiveMode = false;
		active = false;
		//the exact constructors for active elements are defined in their classes
	}
	public UIActiveElement(Coordinate position, int mode){
		super(position);
		toggleActiveMode = false;
		active = false;
		bindMode(mode);
	}
	public void bindMode(int i){
		boundMode = i;
	}
	public int getMode(){
		if(active){
			active = false;
			return boundMode;
		}else{
			return -1;
		}
	}
	public boolean isActive(){
		return active;
	}
	protected void makeActive(){
		if(toggleActiveMode){
			System.out.println("Making "+this + " active!");
			active = true;
			toggleActiveMode = false;
		}
	}
	public void testForActive(CursorInfo inf){
		//this is also on a per-active-element basis. Tests some input to see if this is active
	}
	public void setAppearance(){
		//this lets us decide how the active element appears by default.
	}
	public void setActiveAppearance(){
		//this lets us describe how the element will appear when it is the active one
	}
	public boolean contains(Coordinate c){
		return false;
	}
}