package com.ixee.ui.ogl;

import java.util.ArrayList;
import com.ixee.GraphIX.glEEngine;
import com.ixee.ui.*;
import com.ixee.io.stdio;

//all of the menus override doStuff(), which is the mechanism whereby
//menus actually get things done.

//Menus simply extend the UI, which takes already-defined parts of the game
//and presents them and their relations in more understandable and human-readable
//ways.

//All of the components that the UI uses should have pre-structured and placed,
//because of the aforementioned postulate.

//		Hey so, crazy idea.
//			Make MenuOGL an abstract class.

//		RIGHT?
public class MenuOGL implements AbstractMenu{
	protected ArrayList<UIElement> elements;
	protected ArrayList<UIActiveElement> activeElements;
	protected boolean top;
	protected boolean occludable;
	protected double closeTime;
	protected double closeStartTime;
	protected int mode = -1;
	protected int behaviorOnActiveChange = CHANGE_ONLY_TO_TEXT;
	protected UIRecievesText activeText;
	protected UIActiveElement mostRecentElement;
	protected MenuStackOGL parent = null;

	public static final int CHANGE_ONLY_TO_TEXT = 0;
	public static final int CHANGE_TO_ANY_ELEMENT = 1;

	public MenuOGL(MenuStackOGL m){
		parent = m;
		elements = new ArrayList<UIElement>();
		activeElements = new ArrayList<UIActiveElement>();
		top = false;
		occludable = true;
		closeTime = 0;
		closeStartTime = 0;
	}
	public ArrayList<UIElement> getElements(){
		return elements;
	}
	public boolean isOccludable(){
		return occludable;
	}
	public boolean topMenu(){
		return top;
	}
	public void doStuff(){
		//we do stuff here. Standard menu doesn't.
		//the idea is to use a switch()case statement to decide though.
	}
	public String whatWillThisDo(int i){
		//return a string stating what this method will do for
		//for the given mode.
		return "This is the default menu. It won't do anything for any mode.";
	}
	public int validModes(){
		//returns the number of valid modes this menu has.
		//thus, the valid modes themselves range from 0 to returnable-1
		return activeElements.size();
	}
	public void resize(){
		//this does whatever needs to be done when the window is supposed to be resized.
	}
	public void draw(glEEngine e){
		for(int i=0;i<elements.size();i++){
			elements.get(i).draw(e);
		}
	}
	public boolean checkForMouseOcclusion(CursorInfo listen){
		boolean returnable = false;
		for(int i=0;i<elements.size();i++){

			if(listen.clicked(listen.LEFT) || listen.clicked(listen.RIGHT) || listen.pressed(listen.LEFT) || listen.pressed(listen.RIGHT)){
				//System.out.println(this+", "+(listen.clicked(listen.LEFT) || listen.clicked(listen.RIGHT) || listen.pressed(listen.LEFT) || listen.pressed(listen.RIGHT)));
				if(elements.get(i).contains(listen.position())){
					returnable = true;
				}
			}
		}
		return returnable;
	}
	public boolean check(CursorInfo listen, KeyboardInfo keyboard){
		boolean returnable = checkForMouseOcclusion(listen);
		for(int i=0;i<activeElements.size();i++){
			activeElements.get(i).testForActive(listen);
			if(activeElements.get(i).isActive()){
				if(mostRecentElement != null){
					mostRecentElement.setFocused(false);
				}
				mostRecentElement = activeElements.get(i);
				mostRecentElement.setFocused(true);
				if(mostRecentElement instanceof UIRecievesText){
				//	System.out.println("Is alive: "+mostRecentElement);
					activeText = (UIRecievesText)mostRecentElement;
				}else if(behaviorOnActiveChange == CHANGE_TO_ANY_ELEMENT){
					activeText = null;		//If we change to any element and the element
											//to change to cannot recieve text, set the
											//current text reciever to null.

							//This also gives the menu the chance to act upon any
							//remaining keyboard input.
				}
				mode = activeElements.get(i).getMode();
			}
		}
		boolean getMore = (activeText != null);
		while(keyboard.hasNext() && getMore == true){
		//	stdio.debugAndLog(this, new Character(keyboard.keyAsChar()).toString());
			getMore = activeText.recieveText(keyboard.keyAsChar());	//Text being recieved should
			keyboard.nextKey();										//not have to be anything more
																	//than a letter or control char
		}
		if(keyboard.hasNext()){
			keyboard.nextKey();
			if(keyboard.keyAsChar() == 10){
				if(mostRecentElement != null){
					mode = mostRecentElement.getMode();
				}
			}else if(keyboard.keyAsChar() == 8){
				System.out.println("Tab...!");
				mostRecentElement = activeElements.get((activeElements.indexOf(mostRecentElement)+1)%activeElements.size());
				mode = mostRecentElement.getMode();
			}
		}
		doStuff();
		return returnable;
	}
	public double getCloseDelay(){
		double returnable = 0;
		for(int i=0;i<elements.size();i++){
			returnable = Math.max(elements.get(i).getCloseDelay(), returnable);
		}
		return returnable;
	}
	public boolean closed(){
		return (getCloseDelay() < (System.currentTimeMillis()-closeStartTime));
	}
	public void close(){
		closeStartTime = System.currentTimeMillis();
		for(int i=0;i<elements.size();i++){
			elements.get(i).close();
		}
	}
	public void add(UIElement e){
		elements.add(e);
		if(e instanceof UIActiveElement){
			activeElements.add((UIActiveElement)e);
		}
	}
}