/*
	Slider simply is a generic |-------[]--| type element. It is described in terms of length and thickness (no jokes, please.)

	The thickness of the inner bar is set externally; the only value given by the slider itself is a double ranging from
		[0,1] describing where the inner bar is. The full range of distance the inner bar can move is length-buttonThickness.

	Two methods to resize:

		resize(double d) sets the bar to some length d and preserves the inner bar's position by rescaling the ratio.
			ex: bar is 10 long, ratio is 0.5, then is resized to 20 long. The ratio is then scaled down so length+ratio=constant

		rescale(double d) sets the bar to some length d and does not change the inner bar's position

*/

package com.ixee.ui.ogl;

import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.*;
import com.ixee.ui.CursorInfo;
import com.ixee.math.Geometry;
import java.awt.Color;

public class UISlider extends UIActiveElement{
	UIRect model = new UIRect(new Coordinate(0,0),new Coordinate[]{new Coordinate(0,0),new Coordinate(0,0),new Coordinate(0,0),new Coordinate(0,0)}, new Color(0, 0, 128));
	//double internalLength;
	double range = 1;			//the total range of the whole thing. Independant from the actual bar size.
	double sampleSize = 1;		//how much of the range the bar can "see"

	double barPos;
	double thickness;
	double length;
	double barSize;
	double incAmount;
	boolean vertical = false;

	Coordinate bounds;

	public UISlider(Coordinate position, Coordinate b, int mode){
		super(position, mode);
		bounds = b;
		thickness = bounds.getX();
		length = bounds.getY();
		boolean setVertical = bounds.getY()>bounds.getX();
		vertical = false;
		if(setVertical){
			rotate();
		}
		model.setFill(true);
		model.setCenter(position);
		model.setCoords(new Coordinate[]{
			new Coordinate(b.getX()/2, b.getY()/2),
			new Coordinate(b.getX()/2, -b.getY()/2),
			new Coordinate(-b.getX()/2, -b.getY()/2),
			new Coordinate(-b.getX()/2, b.getY()/2)
			});
		barSize = sampleSize/range;
		incAmount = sampleSize;
		setBarValue(0);
		bounds = b;
	}
	private void rotate(){
		double newLen = thickness;
		thickness = length;
		length = newLen;
		barSize = length*sampleSize/range;
		vertical = !vertical;
	}
	public Coordinate getBounds(){
		return bounds;
	}
	public void setBounds(Coordinate b){
		bounds = b;
		model.setCoords(new Coordinate[]{
			new Coordinate(b.getX()/2, b.getY()/2),
			new Coordinate(b.getX()/2, -b.getY()/2),
			new Coordinate(-b.getX()/2, -b.getY()/2),
			new Coordinate(-b.getX()/2, b.getY()/2)
			});
	}
	public boolean isVertical(){
		return vertical;
	}
	public void setCenter(Coordinate c){
		super.setCenter(c);
		model.setCenter(c);
	}
	public void setSampleSize(double d){
		sampleSize = d;
		sampleSize = Math.min(range, sampleSize);
		sampleSize = Math.max(0.1, sampleSize);
		barSize = sampleSize/range;
	}
	public double getSampleSize(){
		return sampleSize;
	}
	public double getLength(){
		return length;
	}
	public void setIncrementAmount(double d){
		incAmount = d;
		incAmount = Math.max(0, incAmount);
		incAmount = Math.min(range, incAmount);
	}
	public double getIncrementAmount(){
		return incAmount;
	}
	public void rescale(double d){
		range = d;
		range = Math.max(0, range);
		sampleSize = Math.min(range, sampleSize);
		barSize = sampleSize/range;
		barSize = Math.min(Math.max(barSize, 3), 20);
	}
	public void resize(double d){
		double barOrig = barPos*range;
		range = d;
		range = Math.max(0, range);
		sampleSize = Math.min(range, sampleSize);
		barPos = barOrig/range;
		barSize = sampleSize/range;
		barSize = Math.min(Math.max(barSize, 0.1), 20);
	}
	public double getRange(){
		return range;
	}
	public double getRatio(){
		return (barPos);
	}
	public double getValue(){
		return getRatio()*range;
	}
	public void increment(){
		barPos += incAmount/range;	//incAmount starts at sampleSize. div by range converts it into terms of the slider.
		barPos = Math.min((length-barSize)/length, barPos);
		System.out.println(barPos);
	}
	public void decrement(){
		barPos -= incAmount/range;	//incAmount starts at sampleSize. div by range converts it into terms of the slider.
		barPos = Math.max((barSize)/length, barPos);
		System.out.println(barPos);
	}
	public void setRatio(double d){
		if(d<0)d=0;
		if(d>1)d=1;
		barPos = d;
	}
	public void setBarValue(double d){
		if(d>range)d=range;
		if(d<0)d=0;
		setRatio(d/range);
//		barPos = d;
	}
	public void setSize(Coordinate bounds){
		thickness = bounds.getX();
		length = bounds.getY();
		boolean setVertical = bounds.getY()>bounds.getX();
		vertical = false;
		if(setVertical){
			rotate();
		}
	}
	public boolean contains(Coordinate c){
		boolean returnable = false;
		if(vertical){
			if(Geometry.contains(new Coordinate(thickness, length), c)){
				returnable = true;
			}
		}else{
			if(Geometry.contains(new Coordinate(length, thickness), c)){
				returnable = true;
			}
		}
		return returnable;
	}
	public void draw(glEEngine e){
		model.draw(e);
		double thickness = Math.min(bounds.getX(), bounds.getY());
		if(vertical){
			e.fillRect(new Coordinate(barPos+barSize*bounds.getY()/2, -thickness/2).add(centerpoint), new Coordinate(barPos-barSize*bounds.getY()/2, thickness/2).add(centerpoint), new Color(120, 0, 200));
		}else{
			e.fillRect(new Coordinate(-thickness/2, (barPos-0.5)*bounds.getY()-barSize*bounds.getY()/2).add(centerpoint), new Coordinate(thickness/2, (barPos-0.5)*bounds.getY()+barSize*bounds.getY()/2).add(centerpoint), new Color(120, 0, 200));
		}
	}
	public void testForActive(CursorInfo cursInf){
		if(cursInf.pressed(0)){
			Coordinate pressLoc = cursInf.position().sub(centerpoint);
			double newSliderLoc = Double.NaN;
			if(vertical){
				if(Geometry.contains(new Coordinate(thickness, length), pressLoc)){
					newSliderLoc = pressLoc.getX();
				}
			}else{
				if(Geometry.contains(new Coordinate(length, thickness), pressLoc)){
					newSliderLoc = pressLoc.getY();
				}
			}
			if(!Double.isNaN(newSliderLoc)){
				newSliderLoc/=Math.max(bounds.getX(), bounds.getY());
				newSliderLoc+=0.5;
				//barPos = Math.min(Math.max(0, newSliderLoc), 1);
				setRatio(newSliderLoc);
				active = true;
			}
			//jump the box to there
			//we get smooth tracking either way from this point
		}

	}
	public void setAppearance(){}
	public void setActiveAppearance(){}
	public double right(){
		return centerpoint.getX()+bounds.getX()/2;
	}
	public double left(){
		return centerpoint.getX()-bounds.getX()/2;
	}
	public double top(){
		return centerpoint.getY()+bounds.getY()/2;
	}
	public double bottom(){
		return centerpoint.getY()-bounds.getY()/2;
	}
	public void close(){}
}