package com.ixee.ui.ogl;

import com.ixee.io.stdio;
import java.awt.Color;
import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.*;
import com.ixee.ui.CursorInfo;

public class UITextInput extends UIActiveElement implements UIRecievesText{
	String value;
	String currentTextBuffer;
	UIText displayable;
	Coordinate[] poly;
	Coordinate[] prompt;
	Coordinate bounds;
	int numChars;
	double downscale = 0.9;
	int behaviorOnEnterRecieved = STOP_RECIEVING;

	boolean displayText = true;

	public static final int STOP_RECIEVING = 0;
	public static final int ADD_NEWLINE = 1;

	public UITextInput(Coordinate c, Coordinate bound, int m){
		super(c, m);
		currentTextBuffer = "";
		poly = new Coordinate[]{
			bound.mult(0.5).add(c),
			new Coordinate(bound.getX()/2, -bound.getY()/2).add(c),
			bound.mult(-0.5).add(c),
			new Coordinate(-bound.getX()/2, bound.getY()/2).add(c)
		};
		prompt = new Coordinate[]{
			new Coordinate(c.getX()-bound.getX()/2+8, c.getY()-6),
			new Coordinate(c.getX()-bound.getX()/2, c.getY()-8)
		};
		displayable = new UIText(c.sub(new Coordinate(bound.getX()/2-5, 0)), "", UIText.H_LEFT);
		numChars = (int)bound.getX()/8;
		displayable.setNumChars((int)bound.getX()/8);
		bounds = bound;
	}
	public String getText(){
		return currentTextBuffer;
	}
	public void resetText(){
		currentTextBuffer = "";
		displayable.setText("");
	}
	public void setShowText(boolean b){
		displayText = b;
	}
	public void testForActive(CursorInfo listen){
		Coordinate testPoint = listen.releaseLoc(CursorInfo.LEFT);
		if(listen.clicked(CursorInfo.LEFT) && contains(testPoint)){
			toggleActiveMode = true;
		}
		makeActive();
	}
	public boolean contains(Coordinate c){
		boolean returnable = false;
		double leftBound = centerpoint.getX()-bounds.getX()/2;
		double rightBound = centerpoint.getX()+bounds.getX()/2;
		double lowerBound = centerpoint.getY()-bounds.getY()/2;
		double upperBound = centerpoint.getY()+bounds.getY()/2;
		//System.out.print("Checking for "+c+" in " + leftBound + " "+rightBound+" "+lowerBound+" "+upperBound+"...");
		if(c.getX() > leftBound && c.getX() < rightBound){
			if(c.getY() > lowerBound && c.getY() < upperBound){
				returnable = true;
			}
		}
		//System.out.println(" " + returnable);
		return returnable;
	}
	public boolean recieveText(char more){
		boolean returnable = true;
		if(more == 0){
		}else if(more == 8){
//			stdio.debug(this, "Backspace!");
			currentTextBuffer = currentTextBuffer.substring(0, Math.max(0, currentTextBuffer.length()-1));
		}else if(more == 10){
//			stdio.debug(this, "Enter!");
			if(behaviorOnEnterRecieved == STOP_RECIEVING){
				returnable = false;
			}else if(behaviorOnEnterRecieved == ADD_NEWLINE){
				currentTextBuffer = currentTextBuffer.concat("\n");
			}
		}else if(more != 127 && more != 9){
			//System.out.println((new Integer((int)more)).toString());
			currentTextBuffer = currentTextBuffer.concat(new Character(more).toString());
		}
		if(!displayText){
			displayable.setText(new String(new char[currentTextBuffer.length()]).replace("\0", "*"));
		}else{
			displayable.setText(currentTextBuffer);
		}
		return returnable;
	}
	public void draw(glEEngine e){
		//Perhaps implement some code for a sort of context-based
		//way to draw UIElements;
		/*
				have a setContext() method that moves (0,0) to the centerpoint
				of this UIElement?
		*/
		e.fillRect(poly[0], poly[2], new Color(0.4f, 0.8f, 0.4f, 0.75f));
		displayable.draw(e);
		if(focused){
			e.fillRect(prompt[0].add(new Coordinate(8*Math.min(numChars-1, currentTextBuffer.length())+5, 0)),prompt[1].add(new Coordinate(8*Math.min(numChars-1, currentTextBuffer.length())+5, 0)), new Color(190, 190, 190));
			e.drawRect(poly[0].sub(centerpoint).mult(downscale).add(centerpoint), poly[2].sub(centerpoint).mult(downscale).add(centerpoint), new Color(0.7f, 0.2f, 0.5f, 0.9f));
		}
	}
	public void setAppearance(){}
	public void setActiveAppearance(){}
	public void close(){}
	public double right(){
		return displayable.right();
	}
	public double left(){
		return displayable.left();
	}
	public double top(){
		return displayable.top();
	}
	public double bottom(){
		return displayable.bottom();
	}
}